#include "usart.h"

#include <avr/io.h>
#include <util/delay.h>


#define MORSE_LUNG  3
#define MORSE_SCURT 1
#define AREF 5

void ADC_init(void)
{
    ADMUX = (1 << REFS0) | (1 << REFS1);
    ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

unsigned int ADC_get(uint8_t channel)
{
    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}


int lumina () {
    USART0_print("Aflu lumina!\r\n");
    _delay_ms(10);
    unsigned int rez=ADC_get(0);
    char val[20]; 
    double Rsensor=(double)(1023-rez)*10/rez;
    sprintf(val, "%i lumeni\r\n", (int)Rsensor);
    USART0_print(val);
    _delay_ms(2000);
    if(Rsensor > 100)
        return 0;
    return 1;
}


int umiditate() {
    USART0_print("Aflu umiditatea!\r\n");
    _delay_ms(10);
    unsigned int rez=ADC_get(1);
    char val[20]; 
    double Rsensor=(double)(1023-rez)*50/rez;
    sprintf(val, "%i %%\r\n", (int)Rsensor);
    USART0_print(val);
    _delay_ms(2000);
    if(Rsensor < 20) {
        return 1;
    }
    return 0;
}

int temperatura () {
    USART0_print("Aflu temperatura!\r\n");
        _delay_ms(10);
    /* Setez pinul PB2 ca intrare. */
    
    DDRB &= ~(1 << PB2);
    /* Activez rezistenta de pull-up pentru PB2. */
    
    PORTB |= (1 << PB2);
    int val=(PINB & (1 << PB2));

    char text[20];
    sprintf(text, "coeficient de temperatura=%i\r\n", (int)val);
    USART0_print(text);
    if (val > 0) {
        USART0_print("Temperatura e propice!\r\n");
        return 1;
    }
    else {
        USART0_print("Temperatura nu este propice pentru udat!\r\n");

        return 0;
    }

}
void all () {
    int count=0;
    count+=lumina();
    _delay_ms(100);
    count+=umiditate();
    _delay_ms(100);
    count+=1;
    if(count==3) {
        USART0_print("E timpul pentru udat, hihi!\r\n");
        PORTD |= (1 << PD7);
        _delay_ms(5000);
        PORTD &= ~(1 << PD7);
    }
    else 
        USART0_print("Planta se prezinta in parametrii normali!\r\n");
}

void task_usart()
{
    char c;

        /* Setez pinul PD7 ca iesire. */
        DDRD |= (1 << PD7);
        /* Sting LED-ul. */
        PORTD &= ~(1 << PD7);
        //setez pinul pentru buton
        DDRD &= ~(1 << PD6);
        PORTD |= (1 << PD6);
        ADC_init();
        // main loop
    while(1)
    {
        
        c = USART0_receive();
       
        if (c == 's')
        {
            USART0_print("Am primit start!\r\n");
            _delay_ms(10);
            PORTD |= (1 << PD7);
            _delay_ms(500);
            PORTD &= ~(1 << PD7);
        }
        else if(c == 'x') {
            USART0_print("Am primit stop!\r\n");
            _delay_ms(10);
             PORTD |= (1 << PD7);
            _delay_ms(1000);
            PORTD &= ~(1 << PD7);
            _delay_ms(50);
            break;
        }
        else if(c == 'l') {
            lumina();
        }
        else if(c == 'u') {
            
            umiditate();
        }
        else if (c == 't') {
            temperatura();
            continue;
        }
        //verific daca butonul este apasat
        else if ((PIND & (1 << PD6)) == 0) {
            USART0_print("Ai apasat butonul\r\n");
            _delay_ms(1000);
            all();
        }
        else if (c == 'a') 
            all();
    }
}



int main()
{
    //USART
    USART0_init();
    ADC_init();
    task_usart();
    USART0_print("Am iesit de tot\r\n");
    return 0;
}

